# Data

The Data tab allows users to explore the records associated with a given dataset. The tab includes two main components: 

1. a data table;
2. a map, displayed only for geographical datasets (it makes sense, doesn't it?).

![Data](../../../../assets/webapp/data.png)

The user can toggle the visibility of each of the two components by clicking on the dedicated buttons. The entire tab can also go fullscreen.

## Data table

Each entry of a given dataset is rendered as a line in a table. Data value having simple types (string, integer, float, ...) are fully displayed. Complex data values, such as JSON objects, are replaced by a placeholder signaling the presence of a complex property. Records are retrieved from the search engine in a paginated way (30 items per page), in order not to overload the browser. 

**QUESTION: how is scrolling achieved? Do we use the dedicated Elasticsearch feature?**



### Search within the data entries

A dedicated text input allows the user to perform a full-text search within the entire dataset. The number of matching records is shown and visually illustrated in the guise of a progress bar.

**QUESTION: how are search terms handled? Do we use a + concatenating the various terms?**

### Column visibility

The visibility of each column can be toggled by clicking on a dedicated button and (de)selecting the various checkboxes.

### Interactions with the data table

* Each column can be sorted either alphabetically or numerically, according to the data type detected by the search engine.

* Upon clicking on a given entry on the table, the entry gets fully displayed (including complex properties) on a dedicated viewer. If the map component is visible, then the map gets centered on the clicked entry.

* Already visited entries are highlighted, on the table, with a different background color.


## Map

### Under the hood

The map component is powered by the [Mapbox GL JS](https://github.com/mapbox/mapbox-gl-js) library.

### Layers

Three overlaid layers are used:

1. a base layer, either vector or raster depending of the choice of the user. The raster layer is based on the [orthoimages](https://data.grandlyon.com/en/jeux-de-donnees/orthophotographie-2018-metropole-lyon-format-tiff/donnees) provided by the Métropole de Lyon with the addition of street labels coming from the vector base layer.

2. A raster layer, displaying the entire (geographic) dataset by issuing requests against a Web Map Service (WMS).

3. A transparent vector layer, fed by a Mapbox Vector Tiles (MVT) service, allowing users to interact with the raster features displayed by the WMS. Individual features get highlighted on hover and click events.

#### Restricted-access datasets

Some of the datasets hosted by data.grandlyon.com are only accessible to authorized users. For such datasets, two different behaviors were implemented:

1. for **authorized users**: the very same behavior as for unrestriced datasets, in a seamless way;

2. for **unauthorized users**: the raster layer, powered by the WMS, displays the full dataset; user interaction is limited to a small subset of geographical features and achieved by constructing a GeoJSON out of the sample returned by the search engine.

This feature relies on a custom user-aware component, proxying requests to the Web Mapping Services hosted by the data.grandlyon.com Core.

#### 3D display

This is an emulated 3D, using 2D information. The vector base layer contains the height of the buidings (<- FALSE !). With Mapbox GL JS it is possible to create a layer by setting some style to the polygons with the *fill-extrusion* property.

**QUESTION: which dataset is used?**

### Research by address & Geolocalisation

It is possible to search an address in the map. We use an internal geocoder API based on [Photon](https://github.com/komoot/photon) (open source geocoder built for OpenStreetMap data, based on Elasticsearch).  

It's also possible to center the map to the user position. It uses the native [Navigator API](https://developer.mozilla.org/en-US/docs/Web/API/Navigator/geolocation).

### *Share* feature

This allow with a button click to copy the link of the map with the current context.  
This context contains: the zoom, position, bearing, pitch and base layer.  

The URL created will be the following:
`http://hostName#zoom/latitude/longitude/bearing/pitch/selectedBaseLayerId`
Then when you access the map in a new tab, the application will load and display the map with all the correct settings.

### Interactions

When a user clicks on a feature (point, line, polygon) a window on the right will open and display all the properties and values.

In the same time, this feature will be highlighted in the data table, and set in first position.

### Styling

The OpenMapTiles basemap [Klokantech Basic](https://openmaptiles.org/styles/#klokantech-basic) has been used to help us create our style.  
This style (and all of the Open Map Tiles styles) use the [Mapbox GL style specification](https://docs.mapbox.com/mapbox-gl-js/style-spec/) to define the visual appearance of a map.

#### Sprites

The map uses some custom sprites, as the grass in green area, so it was needed to create and host custom sprites. The worflow for this has been the following:

* download the [Mapbox icons](https://labs.mapbox.com/maki-icons/) (the Maki icons)
* add the custom SVG we want to use in our style (for example grass.svg and water.svg)
* clone and install [spritezero-cli](https://github.com/mapbox/spritezero-cli), a command-line interface for [spritezero](https://github.com/mapbox/spritezero)
* `spritezero [output filename] [input directory]` . It generates a JSON layout file and the associated PNG spritesheet.
Also generate it with the `--retina` option to have the @2x sprites
* Once the folder containing these files is hosted, we reference it in the style file:  
`"sprites": 'path/to/the/sprites/folder/'`
* Finally we can style the custom grass in green area:

```json
"paint": {
        "fill-pattern": "grass"
      }
```

#### Glyphs

We wanted to have in the map the same font we use in the web app: the [Titillium Web](https://fonts.google.com/specimen/Titillium+Web). In Mapbox GL the fonts are called Glyphs and are rendered as SDFs (signed distance fields). It makes it easier than regular font for rotation, scaling..etc.  
To convert our `.ttf` into protobuf-encoded SDF glyphs we use [node-fontnik](https://github.com/mapbox/node-fontnik).  
Once the project cloned, we can

```bash
npm install
node-fontnik/bin/build-glyphs font.ttf /glyph-folder-pbf
```

Finally you can reference it in your style file as the following:  
`"glyphs": 'path/to/the/glyphs/folder/'`  

We have to do the conversion one by one. If multiple `.ttf` have to be converted, then maybe a `.sh` script is better.  
In our case we needed only `Titillium Web` to add in our map glyphs. If another font is needed and you want to use them in a single style file, then you need to combine them. For this you can use [glyph-pbf-composite](https://github.com/mapbox/glyph-pbf-composite)


**TO ADD: "bannière rouge"**